// Use shared memory

// Include
#include "xgc_graphics.h"
#include <stdio.h>
#include <string.h>
#include <print.h>

// Extern vars
extern char screen[MAX_LINES][MAX_CHARS_LINE];

static int line_pointer=0;
static int char_pointer=0;

// Set some routines

// Clear the screen
void graphics_clear_screen() {
	memset((void *) screen, 0, MAX_LINES * MAX_CHARS_LINE);
	line_pointer=0;
	char_pointer=0;
}

// Add a char to the screen, wrapping on to next line when required
void graphics_add_char(char ch) {
	if (char_pointer==MAX_CHARS_LINE) {
		graphics_new_line();
	}
	
	screen[line_pointer][char_pointer] = ch;
	char_pointer++;
}

// Add a line of text to the screen
void graphics_add_text(char chs[]) {
	int i;
	for (i=0;i<strlen(chs);i++) {
		graphics_add_char(chs[i]);
	}
}

// Set line to text
void graphics_set_line(char chs[], int line) {
	strncpy(chs, screen[MAX_LINES], MAX_CHARS_LINE);
}

// Replace previous char
void graphics_del_prev_char() {
	if (char_pointer!=0) {
		screen[line_pointer][char_pointer-1] = 0;
		char_pointer--;
	} else {
		if (line_pointer!=0) {
			line_pointer--;
			char_pointer = MAX_CHARS_LINE;
		
			screen[line_pointer][char_pointer] = 0;
		}
	}
}

// Add a new line to the screen
void graphics_new_line() {
	if (line_pointer<MAX_LINES-1) {
		line_pointer++;
		char_pointer=0;
	} else {
		int i=1;
		// Need to shift everything up
		for (i=1; i<MAX_LINES; i++) {
			memcpy(screen[i-1], screen[i], MAX_CHARS_LINE);
		}
		memset(screen[MAX_LINES-1], 0, MAX_CHARS_LINE);
		line_pointer=MAX_LINES-1;
		char_pointer=0;
	}
}

// Check position of cursor
int graphics_cursor_pos_lines() {
	return line_pointer;
}
int graphics_cursor_pos_chars() {
	return char_pointer;
}

// Set position of cursor
void graphics_cursor_set_line(int li) {
	line_pointer = li;
}
void graphics_cursor_set_char(int ch) {
	char_pointer = ch;
}