#ifndef XGC_SERVICES_H_
#define XGC_SERVICES_H_

// Kernel

// Includes
#include "FAT16_client.h"
#include "xgc_sdcard.h"
#include "xgc_editor.h"
#include "xgc_graphics.h"

// Internal funcs
void xgc_services(chanend services_kbd_chars, streaming chanend services_kbd_req, chanend services_sd, chanend services_shell);
void request_handler(int request, chanend app, chanend services_kbd_chars, streaming chanend services_kbd_req, chanend services_sd);

void ak_get_input_status(chanend app);
void ak_key_setmode(chanend app);
void ak_key_req(chanend services_kbd_chars, streaming chanend services_kbd_req, chanend app);
void ak_key_nbreq(chanend services_kbd_chars, streaming chanend services_kbd_req, chanend app);
void ak_graphics_add_char(chanend app);
void ak_graphics_del_prev_char();
void ak_graphics_new_line();
void ak_graphics_cs();
void ak_graphics_set_line(chanend app);
void ak_graphics_set_char(chanend app);
void ak_sdcard_read(chanend app, chanend services_sd);
void ak_sdcard_write(chanend app, chanend services_sd);
void ak_sdcard_del(chanend app, chanend services_sd);
void ak_sdcard_inserted(chanend app, chanend services_sd);
void ak_sdcard_exists(chanend app, chanend services_sd);
void ak_sdcard_cat(chanend app, chanend services_sd);
void ak_sdcard_cat(chanend app, chanend services_sd);
void ak_speaker_tone(chanend app);

// Public funcs
extern void services_graphics_add_char(chanend from_services, char ch);
extern void services_graphics_new_line(chanend from_services);
extern void services_graphics_add_text(chanend from_services, char array[]);
extern void services_graphics_set_line(chanend from_services, int pos);
extern void services_graphics_set_char(chanend from_services, int pos);
extern void services_graphics_cs(chanend from_services);
extern int services_sdcard_load(chanend from_services, char filename[], char dest[], int dest_size);
extern int services_sdcard_write(chanend from_services, char filename[], char src[], int src_size);
extern int services_sdcard_del(chanend from_services, char filename[]);
extern int services_sdcard_inserted(chanend from_services);
extern int services_sdcard_exists(chanend from_services, char filename[]);
extern int services_sdcard_cat(chanend from_services, char cat[], int cat_len);


// Uility public functions
extern void itoa(int n, char s[]);
extern void substr(char src[], char dest[], int start, int end, int stripspace);

// Defines

// This is the protocol for communication between the kernel and processes
#define KA_SYS_WAKEUP 1			// Wakeup application
#define AK_SYS_FINISHED 18		// Application says it has finished
#define AK_SYS_GIVE_CONTROL 19	// Application can give control to another thread
#define AK_KEY_SETMODE 26		// Set the keyboard mode, either 0 blocking or 1 non-blocking
#define AK_KEY_REQ 2			// Blocking key request
#define AK_KEY_NBREQ 25			// Non-blocking key request
#define AK_GRAPHICS_ADD_CHAR 4
#define AK_GRAPHICS_NEW_LINE 5
#define AK_GRAPHICS_DEL_PREV_CHAR 7
#define AK_GRAPHICS_CS 20		// Clear screen
#define AK_GRAPHICS_SET_LINE 21 // Set line
#define AK_GRAPHICS_SET_CHAR 22 // Set char
#define KA_SDCARD_OKAY 8		// SD card operations need error checking, indicate OK
#define KA_SDCARD_ERROR 9		// Next is the error number
#define AK_SDCARD_CAT 10		// Data follows is array of characters, list of files separated by ' '
#define AK_SDCARD_READ 11		// Data follows is array of characters (or SDCARD_ERROR)
#define AK_SDCARD_WRITE 12		// Follows is size and then data character by character (or SDCARD_ERROR)
#define AK_SDCARD_DEL 17		// Delete
#define AK_SDCARD_INSERTED 13	// Is a card inserted? Returns OKAY or ERROR
#define AK_SDCARD_EXISTS 15		// Takes stream and returns OKAY or ERROR depending on if file exists
#define AK_SPEAKER_TONE 23		// Output tone to speaker, expects len (1/10 sec) and hz
#define AK_GET_INPUT_STATUS 24	// Returns a 32 bit integer with bits corresponding to the status of the buttons and joystick

// Service constraints
#define MAX_SD_READ 3000
#define MAX_SD_WRITE 3000

#endif /*XGC_SERVICES_H_*/
