// Define commands for a simple shell

// Includes
#include "xgc_shell.h"
#include "xgc_interpreter.h"

// Main function
void shell(chanend from_services) {
	while (1) {
		
		// When the shell first starts, output the prompt
		services_graphics_new_line(from_services);
		services_graphics_add_text(from_services, "welcome to Xshell");
		services_graphics_new_line(from_services);
		
		// Start a loop to accept commands
		while (1) {
			char command[MAX_CMD_LEN+1];
			char command_old[MAX_CMD_LEN+1];
			int cmd_next_char=0;
			int cursor=0;
			int curr=0;
			
			// Set array
			safememset(command, 0, MAX_CMD_LEN+1);
			
			// Print preamble
			services_graphics_new_line(from_services);
			services_graphics_add_char(from_services, 0x05);
			services_graphics_add_char(from_services, 0x20);
			services_graphics_add_char(from_services, '_');
			
			// While the user is inputting a command
			while (1) {
				
				from_services <: AK_KEY_REQ;
				from_services :> curr;

				// Add chars
				if (curr==KEY_ENTER) {
					// It's cosmetic, but redraw without the cursor
					sh_redraw_command(from_services, command, cmd_next_char, cursor, 0);
					break;
				} else {
					
					// Process cursor keys, backspace
					switch (curr) {
						case KEY_ESC:
							// Esc will clear command
							cmd_next_char=0;
							cursor=0;
							command[cmd_next_char]='\0';
							break;
							
						case KEY_BACK:
							if (cursor!=0) {
								// Backspace it
								sh_backspace_char(command, cmd_next_char, cursor);
							}
							break;
							
						case KEY_DEL:
							// For delete, inc cursor and then backspace
							if (cursor!=cmd_next_char) {
								// In cursor
								cursor++;
								
								// Backspace
								sh_backspace_char(command, cmd_next_char, cursor);
							}
							break;
							
						case KEY_LEFT:
							if (cursor!=0) {
								cursor--;
							}
							break;
							
						case KEY_RIGHT:
							if (cursor!=cmd_next_char) {
								cursor++;
							}
							break;	
						case KEY_UP:
							// Overwrite current command with the old one
							safestrcpy(command, command_old);
							
							// Seeeeet
							cmd_next_char = safestrlen(command)+1;
							cursor =  safestrlen(command);
							break;
						case KEY_DOWN:
							// No function
							break;
						case KEY_TAB:
							// Do nothing
							break;
						default:
							// Otherwise process the char
							if ((curr!=0) && (cmd_next_char<MAX_CMD_LEN)) {
								// Insert a char at the cursor position
								sh_insert_char(command, curr, cmd_next_char, cursor);
							}
							break;
					}
					// Redraw
					sh_redraw_command(from_services, command, cmd_next_char, cursor, 1);
				}

				curr=0;
			}
	
			// User didn't enter anything
			if (cmd_next_char==0) continue;
			
			// Process built in commands and pass others to interpreter
			command[cmd_next_char] = '\0';
			graphics_new_line();
			
			// First, store a backup
			safestrcpy(command_old, command);
			
			// Process
			if (command[0]=='/') {
				// Command is system
				shell_commands(command, from_services);
			} else {
				// Interpreted commands
				interpreter_embed(from_services, command);
			}
		}
	}
}

// Redraw the command line
void sh_redraw_command(chanend from_services, char command[], int &cmd_next_char, int &cursor, int show_cursor) {
	int i;
	
	// Set to start
	services_graphics_set_char(from_services, 2);
	
	// Blank the line
	for (i=0; i<MAX_CMD_LEN; i++) {
		services_graphics_add_char(from_services, ' ');
	}
	
	// Now readd the chars
	services_graphics_set_char(from_services, 2);
	for (i=0; i<=cmd_next_char; i++) {
		if ((i==cursor) && (show_cursor)) services_graphics_add_char(from_services, '_');
		if (i!=cmd_next_char) services_graphics_add_char(from_services, command[i]);
	}
}

// Insert a character
void sh_insert_char(char command[], char ch, int &cmd_next_char, int &cursor) {
	int i;
	
	for (i=cmd_next_char; i>=cursor; i--) {
		command[i+1] = command[i];
	}
	
	command[cursor] = ch;
	
	// Inc
	cmd_next_char++;
	cursor++;
}

// Delete a character
void sh_backspace_char(char command[], int &cmd_next_char, int &cursor) {
	int i;
	
	// Shift everything from the cursor position back one
	for (i=cursor; i<=cmd_next_char; i++) {
		// Shift
		command[i-1] = command[i];
	}
	
	// Dec
	cmd_next_char--;
	cursor--;
}

// Parse system commands
void shell_commands(char command[], chanend from_services) {
	if (safestrncmp(command, "/sdcheck", 8)==0) sdcheck(from_services);
	else if (safestrncmp(command, "/sdcat", 6)==0) sdcat(from_services);
	else if (safestrncmp(command, "/sdmore", 7)==0) sdmore(from_services, command);
	else if (safestrncmp(command, "/sdprint", 8)==0) sdprint(from_services, command);
	else if (safestrncmp(command, "/sddel", 6)==0) sddel(from_services, command);
	else if (safestrncmp(command, "/sdtouch", 6)==0) sdtouch(from_services, command);
	else if (safestrncmp(command, "/sdinterpret", 12)==0) sdinterpret(from_services, command);
	else if (safestrncmp(command, "/cs", 3)==0) services_graphics_cs(from_services);
	else if (safestrncmp(command, "/editor", 7)==0) cmdeditor(from_services, command);
	else if (safestrncmp(command, "/clearistate", 12)==0) reset_state();
	else services_graphics_add_text(from_services, "Unknown system command");
}

// System command functions

// Run the editor
void cmdeditor(chanend from_services, char command[]) {
	char filename[MAX_CMD_LEN+1];
	
	// CS
	services_graphics_cs(from_services);
	
	// Extract the filename
	substr(command, filename, 8, MAX_CMD_LEN, 1);
	
	// Run
	editor(from_services, filename);
	
	// CS
	services_graphics_cs(from_services);
}

// Check
void sdcheck(chanend from_services) {
	services_graphics_add_text(from_services, "Checking, please wait.");
	services_graphics_new_line(from_services);
	services_graphics_add_text(from_services, "Do not touch the SD card slot.");
	services_graphics_new_line(from_services);
	services_graphics_new_line(from_services);
	if (services_sdcard_inserted(from_services)==0) {
		services_graphics_add_text(from_services, "SD card inserted");
	} else {
		services_graphics_add_text(from_services, "SD card not inserted");
	}
}
 
// Catalogue
void sdcat(chanend from_services) {
	char cat[MAX_SD_READ+1];
	int rtn=0;
	int i;
	
	// Get cat
	rtn = services_sdcard_cat(from_services, cat, MAX_SD_READ);
	
	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 5:
			services_graphics_add_text(from_services, "Problem finding root directory.");
			break;
		default:
			for (i=0;i<safestrlen(cat);i++) {
				if (cat[i]==0x20) {
					services_graphics_new_line(from_services);
				} else {
					services_graphics_add_char(from_services, cat[i]);
				}
			}
			break;
	}
}

// More
void sdmore(chanend from_services, char command[]) {
	char filename[MAX_CMD_LEN+1];
	char buffer[MAX_SD_READ+1];
	int rtn=0;
	int i;
	
	// Extract the filename
	substr(command, filename, 8, MAX_CMD_LEN, 1);
	
	// Check
	if (safestrlen(filename)==0) {
		services_graphics_add_text(from_services, "Please provide a filename.");
		return;
	} else if (safestrlen(filename)>20) {
		services_graphics_add_text(from_services, "Filename is too long (20chars max).");
		return;
	}
	
	// Try to get
	rtn = services_sdcard_load(from_services, filename, buffer, 3000);

	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 2:
			services_graphics_add_text(from_services, "Unable to access file ");
			services_graphics_add_text(from_services, filename);
			break;
		case 3:
			services_graphics_add_text(from_services, "Unable to read file ");
			services_graphics_add_text(from_services, filename);
			break;
		case 4:
			services_graphics_add_text(from_services, "Filename too long");
			break;
		default:
			for (i=0;i<safestrlen(buffer);i++) {
				if (buffer[i]==0xA) {
					services_graphics_new_line(from_services);

				} else {
					services_graphics_add_char(from_services, buffer[i]);
				}
			}
			break;
	}
}

// Prints to a file called SDPRINT
void sdprint(chanend from_services, char command[]) {
	char buffer[MAX_CMD_LEN+1];
	int rtn=0;
	int i;
	
	// Extract the text
	substr(command, buffer, 9, MAX_CMD_LEN, 1);
	
	// Check
	if (safestrlen(buffer)==0) {
		services_graphics_add_text(from_services, "Nothing to sdprint");
		return;
	}
	
	// Delete if exists
	services_sdcard_del(from_services, SDPRINT_NAME);
	
	// Try to get
	rtn = services_sdcard_write(from_services, SDPRINT_NAME, buffer, safestrlen(buffer));

	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 2:
			services_graphics_add_text(from_services, "Unable to access print file ");
			graphics_add_text(SDPRINT_NAME);
			break;
		case 3:
			services_graphics_add_text(from_services, "Unable to read print file ");
			graphics_add_text(SDPRINT_NAME);
			break;
		case 4:
			services_graphics_add_text(from_services, "Filename too long");
			break;
		default:
			for (i=0;i<safestrlen(buffer);i++) {
				if (buffer[i]==0xA) {
					services_graphics_new_line(from_services);
				} else {
					services_graphics_add_char(from_services, buffer[i]);
				}
			}
			break;
	}
}

// SDinterpret
// Similar to SDmore
void sdinterpret(chanend from_services, char command[]) {
	char filename[MAX_CMD_LEN+1];
	char buffer[MAX_SD_READ+1];
	int rtn=0;
	
	// Extract the filename
	substr(command, filename, 13, MAX_CMD_LEN, 1);
	
	// Check
	if (safestrlen(filename)==0) {
		services_graphics_add_text(from_services, "Please provide a filename.");
		return;
	} else if (safestrlen(filename)>20) {
		services_graphics_add_text(from_services, "Filename is too long (20chars max).");
		return;
	}
	
	// Try to get
	rtn = services_sdcard_load(from_services, filename, buffer, 3500);

	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 2:
			services_graphics_add_text(from_services, "Unable to access file ");
			services_graphics_add_text(from_services, filename);
			break;
		case 3:
			services_graphics_add_text(from_services, "Unable to read file ");
			services_graphics_add_text(from_services, filename);
			break;
		case 4:
			services_graphics_add_text(from_services, "Filename too long");
			break;
		default:
			services_graphics_cs(from_services);
			interpreter_full(from_services, buffer);
			services_graphics_cs(from_services);
			break;
	}
}

// Deletes a file
void sddel(chanend from_services, char command[]) {
	char filename[MAX_CMD_LEN+1];
	int rtn=0;
	
	// Extract the filename
	substr(command, filename, 7, MAX_CMD_LEN, 1);
	
	// Check
	if (safestrlen(filename)==0) {
		services_graphics_add_text(from_services, "Please provide a filename.");
		return;
	} else if (safestrlen(filename)>20) {
		services_graphics_add_text(from_services, "Filename is too long (20chars max).");
		return;
	}
	
	// Delete if exists
	rtn = services_sdcard_del(from_services, filename);
	
	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 2:
			services_graphics_add_text(from_services, "Cannot find file ");
			services_graphics_add_text(from_services, filename);
			break;
		case 4:
			services_graphics_add_text(from_services, "Filename too long");
			break;
		default:
			services_graphics_add_text(from_services, "Deleted ");
			services_graphics_add_text(from_services, filename);
			break;
	}
}

// Sees if a file exists
void sdtouch(chanend from_services, char command[]) {
	char filename[MAX_CMD_LEN+1];
	int rtn=0;
	
	// Extract the filename
	substr(command, filename, 9, MAX_CMD_LEN, 1);
	
	// Check
	if (safestrlen(filename)==0) {
		services_graphics_add_text(from_services, "Please provide a filename.");
		return;
	} else if (safestrlen(filename)>20) {
		services_graphics_add_text(from_services, "Filename is too long (20chars max).");
		return;
	}
	
	// Delete if exists
	rtn = services_sdcard_exists(from_services, filename);
	
	switch (rtn) {
		case 1:
			services_graphics_add_text(from_services, "SD card not inserted");
			break;
		case 2:
			services_graphics_add_text(from_services, "File does not exist - ");
			services_graphics_add_text(from_services, filename);
			break;
		case 4:
			services_graphics_add_text(from_services, "Filename too long");
			break;
		default:
			services_graphics_add_text(from_services, "File exists - ");
			services_graphics_add_text(from_services, filename);
			break;
	}
}
