// VGA Text Driver for use on XGC board.

// Includes
#include <xs1.h>
#include <platform.h>
#include <print.h>
#include "xgc_vga_interface.h"

// Ports
out buffered port:32 pixels = XS1_PORT_8A; 
out port vsync = XS1_PORT_1N;
out port hsync = XS1_PORT_1M;
port clk_port = XS1_PORT_1C;
clock clk_sync = XS1_CLKBLK_1;
clock clk_data = XS1_CLKBLK_2;

// Externs
char screen[MAX_LINES][MAX_CHARS_LINE];
extern unsigned char font[];

// Lookup table, 4 bit to 32 bit
int lookup[16] = {
		0x00000000,
		0xFF000000,
		0x00FF0000,
		0xFFFF0000,
		0x0000FF00,
		0xFF00FF00,
		0x00FFFF00,
		0xFFFFFF00,
		0x000000FF,
		0xFF0000FF,
		0x00FF00FF,
		0xFFFF00FF,
		0x0000FFFF,
		0xFF00FFFF,
		0x00FFFFFF,
		0xFFFFFFFF,
};

void vga_refresh() {
	int x, y, z, i;
	unsigned time;
	
	int ichar;
	
	// Set clock speed for sync clock and set it to output to a 1bit port
    set_clock_div(clk_sync, CLKDIV/2); // 100MHz / (4/2)*2 as *2 is done by the h/w => 25Mhz
    configure_port_clock_output(clk_port, clk_sync);
    
    // Data clock is sourced from the inverted port which the clock is output
    // Hence clock is shifted by 1/2T
    set_port_inv(clk_port);
    configure_clock_src(clk_data, clk_port);
    
    // Associate ports with clocks
	configure_out_port_no_ready(hsync, clk_sync, 0);
    configure_out_port_no_ready(vsync, clk_sync, 0);
    configure_out_port_no_ready(pixels, clk_data, 0);
     
    // Set initials
    vsync <: 1;
    hsync <: 1;
    pixels <: 0;
   
    // Start the two clocks, note data before sync
    start_clock(clk_data);
    start_clock(clk_sync);
    
    // Set time
    time = 10;
    
    while (1) {
    	// Lines of chars
        for (y = 0; y < MAX_LINES; y++) {
        		
            // Rows of chars
			#pragma loop unroll(8)
            for (z = 0; z < 8; z++) {
                
                // Output row z of each char
            	// Do the first char first on the tick of time
            	ichar = font[(screen[y][0]*8)+z];
            	
                pixels @ time <: lookup[(ichar>>4) & 0x0F];
                pixels <: lookup[ichar & 0x0F];
                
                time =time+XPIXELS;
                
                // Do the rest
                for (x = 1; x < MAX_CHARS_LINE; x++) { 
                    // Get char
                    ichar = font[(screen[y][x]*8)+z];

                    // Output
                    pixels <: lookup[(ichar>>4) & 0x0F]; 
                    pixels <: lookup[ichar & 0x0F]; 
                }

                // Set to zero
    			pixels <: 0;
    		
				// After front porch, set hsync to zero
				time = time+HSYNC_FRONT_PORCH;
				hsync @ time <: 0;
			
				// Wait for for sync interval then set high agin
				time = time+HSYNC_SYNC_PULSE;
			
    			// Set Hsync high
				hsync @ time <: 1;
			
    			// Wait for back porch and then start data transfer
				time = time+HSYNC_BACK_PORCH;
            }
        }
        
    	// Wait for data time that should have happened
        time =time+XPIXELS;
         
    	// V sync front porch, wait 10 lines
    	for (i=0; i<VSYNC_FRONT_PORCH-1; i++) {
    	   // After front porch, set hsync to zero
           time = time+HSYNC_FRONT_PORCH;
           hsync @ time <: 0;
            
           // Wait for for sync interval then set high agin
           time = time+HSYNC_SYNC_PULSE;
            
           // Set Hsync high
           hsync @ time <: 1;
            
           // Wait for back porch and then start data transfer
           time = time+HSYNC_BACK_PORCH;
           
           // Wait for data time
           time =time+XPIXELS;
    	}
    	
    	// Pull down VSYNC
    	vsync @ time <: 0;
    	
    	// Wait 2 lines and then pullup
    	for (i=0; i<VSYNC_SYNC_PULSE; i++) {
           // After front porch, set hsync to zero
           time = time+HSYNC_FRONT_PORCH;
           hsync @ time <: 0;
            
           // Wait for for sync interval then set high agin
           time = time+HSYNC_SYNC_PULSE;
            
           // Set Hsync high
           hsync @ time <: 1;
            
           // Wait for back porch and then start data transfer
           time = time+HSYNC_BACK_PORCH;
           
           // Wait for data time
           time =time+XPIXELS;
        }
    	vsync @ time <: 1;
    	
    	// Now wait 31 lines and then start data
        for (i=0; i<VSYNC_BACK_PORCH; i++) {
           // After front porch, set hsync to zero
           time = time+HSYNC_FRONT_PORCH;
           hsync @ time <: 0;
            
           // Wait for for sync interval then set high agin
           time = time+HSYNC_SYNC_PULSE;
            
           // Set Hsync high
           hsync @ time <: 1;
           
           // Wait for back porch and then start data transfer
           time = time+HSYNC_BACK_PORCH;
           
           // Wait for data time
           time =time+XPIXELS;
        }
        
        // Subtract data time that didn't happen
        time -= XPIXELS;
    }
}
